package com.gree.mall.manager.logic.contract;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.contract.ContractTemplateBean;
import com.gree.mall.manager.bean.contract.ContractTemplateVO;
import com.gree.mall.manager.bean.contract.ContractWebsitBean;
import com.gree.mall.manager.commonmapper.CommonMapper;
import com.gree.mall.manager.commonmapper.WebsitMapper;
import com.gree.mall.manager.enums.ContractTypeEnum;
import com.gree.mall.manager.enums.IsYesNoEnum;
import com.gree.mall.manager.enums.base.BaseEnum;
import com.gree.mall.manager.enums.contract.FieldTypeEnum;
import com.gree.mall.manager.enums.contract.SignContractStatusEnum;
import com.gree.mall.manager.enums.material.StateEnum;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.*;
import com.gree.mall.manager.plus.service.*;
import com.gree.mall.manager.utils.fadada.Auth4FDDUtil;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import com.gree.mall.manager.zfire.util.FieldUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class ContractTemplateLogic {
    private final CommonLogic commonLogic;
    private final CommonMapper commonMapper;
    private final ContractTemplateService contractTemplateService;
    private final ContractWebsitService contractWebsitService;
    private final ContractTemplateFieldService contractTemplateFieldService;
    private final WebsitMapper websitMapper;
    private final WebsitContractSignRecordService websitContractSignRecordService;
    private final WebsitContractSignItemService websitContractSignItemService;
    private final WebsitContractSignFieldService websitContractSignFieldService;

    public IPage<ContractTemplateVO> list(ZfireParamBean zfireParamBean) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        FieldUtils.supplyParam(zfireParamBean, ContractTemplateVO.class, adminUser);
        return commonMapper.contractTemplatePage(new Page(zfireParamBean.getPageNum(), zfireParamBean.getPageSize()), zfireParamBean);
    }

    public ContractTemplateBean detail(String contractTemplateId) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        final ContractTemplate template = contractTemplateService.getById(contractTemplateId);
        ContractTemplateBean bean = new ContractTemplateBean();
        BeanUtils.copyProperties(template, bean);
        // 查询已选子网点
        List<ContractWebsitBean> websitBeanList = commonMapper.contractWebsitList(contractTemplateId);
        bean.setItems(websitBeanList);

        if (CollectionUtil.isNotEmpty(bean.getItems()) && template.getContractType().equals(ContractTypeEnum.MAIN.getKey())) {
            final List<String> subWebsitIds = bean.getItems().stream().map(ContractWebsitBean::getSubWebsitId).collect(Collectors.toList());
            List<WebsitContractSignRecord> recordList = websitMapper.querySignStatus(contractTemplateId, subWebsitIds);
            // 签约记录为空new空list
            if (CollectionUtil.isEmpty(recordList)) {
                recordList = new ArrayList<>();
            }
            final Map<String, WebsitContractSignRecord> recordMap = recordList.stream().collect(Collectors.toMap(v -> v.getParentId() + v.getWebsitId(), Function.identity()));
            for (ContractWebsitBean item : bean.getItems()) {
                if (!recordMap.containsKey(item.getParentId() + item.getSubWebsitId())) {
                    item.setSignStatus(SignContractStatusEnum.WAIT_SIGN.getRemark());
                    continue;
                }
                final WebsitContractSignRecord record = recordMap.get(item.getParentId() + item.getSubWebsitId());

                String status = BaseEnum.keyToValueOpt(SignContractStatusEnum.class, record.getStatus()).orElse(SignContractStatusEnum.WAIT_SIGN.getRemark());
                item.setSignStatus(status);
            }
        }

        final List<ContractTemplateField> fields = contractTemplateFieldService.lambdaQuery()
                .eq(ContractTemplateField::getContractTemplateId, contractTemplateId)
                .list();
        bean.setFieldItems(fields);

        return bean;
    }

    public IPage<ContractWebsitBean> queryRelaWebsit(String contractTemplateId, String websitId, String websitName,
                                                     Integer pageNum, Integer pageSize) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        IPage<ContractWebsitBean> page = commonMapper.querySettleRelaWebsit(new Page<>(pageNum, pageSize), contractTemplateId, websitId, websitName);
        return page;
    }

    @Transactional
    public void add(ContractTemplateBean contractTemplate) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        if (adminUser.getType().equals(2)) {
            throw new RemoteServiceException("平台账号禁止操作");
        }
        contractTemplate.setCompanyWechatId(adminUser.getCompanyWechatId())
                .setCompanyWechatName(adminUser.getCompanyName());

        this.validRequireParams(contractTemplate);
        // 检查二级网点的主合同有没有
        this.checkMainDate(contractTemplate);

        String urlWw = contractTemplate.getContractTemplatePdf();
        String templateId = IdWorker.getIdStr();

        Auth4FDDUtil.uploadTemplate(templateId, null, urlWw);

        contractTemplate.setFadadaTemplateId(templateId)
                .setContractTemplateName(contractTemplate.getContractTemplateName().trim())
                .setNeedPartyASign(StringUtils.isBlank(contractTemplate.getCompanySignFiled()) ? IsYesNoEnum.NO.getKey() : IsYesNoEnum.YES.getKey())
                .setStatus(StateEnum.ON.getKey())
                .insert();

        if (CollectionUtil.isNotEmpty(contractTemplate.getItems())) {
            contractWebsitService.saveBatch(contractTemplate.getItems().stream()
                    .peek(v -> v.setContractTemplateId(contractTemplate.getContractTemplateId())
                            .setCompanyWechatId(contractTemplate.getCompanyWechatId())
                            .setCompanyWechatName(contractTemplate.getCompanyWechatName())
                            .setId(IdWorker.getIdStr()))
                    .collect(Collectors.toList()));
        }

        if (CollectionUtil.isNotEmpty(contractTemplate.getFieldItems())) {
            contractTemplateFieldService.saveBatch(contractTemplate.getFieldItems().stream()
                    .peek(v -> v.setContractTemplateId(contractTemplate.getContractTemplateId())
                            .setId(IdWorker.getIdStr()))
                    .collect(Collectors.toList()));
        }

        // 检查附件所属合同是否有签约记录
        if (contractTemplate.getContractType().equals(ContractTypeEnum.FOLLOW.getKey())
                && StringUtils.isNotBlank(contractTemplate.getFollowContractTemplateId())) {
            final List<WebsitContractSignItem> signItems = websitContractSignItemService.lambdaQuery()
                    .eq(WebsitContractSignItem::getContractTemplateId, contractTemplate.getFollowContractTemplateId())
                    .in(WebsitContractSignItem::getContractStatus, SignContractStatusEnum.WAIT_SIGN.getKey(), SignContractStatusEnum.WAIT.getKey(), SignContractStatusEnum.OK.getKey())
                    .groupBy(WebsitContractSignItem::getWebsitContractSignRecordId)
                    .list();

            if (CollectionUtil.isNotEmpty(signItems)) {
                final List<WebsitContractSignRecord> signRecords = websitContractSignRecordService.lambdaQuery()
                        .select(WebsitContractSignRecord::getId)
                        .in(WebsitContractSignRecord::getId, signItems.stream()
                                .map(WebsitContractSignItem::getWebsitContractSignRecordId)
                                .collect(Collectors.toList()))
                        .in(WebsitContractSignRecord::getStatus, SignContractStatusEnum.WAIT.getKey(), SignContractStatusEnum.OK.getKey())
                        .list();
                final Set<String> recordSet = signRecords.stream().map(WebsitContractSignRecord::getId).collect(Collectors.toSet());
                List<WebsitContractSignItem> signItemList = new ArrayList<>();
                List<WebsitContractSignField> signFieldList = new ArrayList<>();
                for (WebsitContractSignItem signItem : signItems) {
                    if (CollectionUtil.isEmpty(recordSet)) {
                        continue;
                    }
                    if (!recordSet.contains(signItem.getWebsitContractSignRecordId())) {
                        continue;
                    }
                    WebsitContractSignItem contractSignItem = new WebsitContractSignItem();
                    contractSignItem.setId(IdWorker.getIdStr())
                            .setWebsitContractSignRecordId(signItem.getWebsitContractSignRecordId())
                            .setContractName(contractTemplate.getContractTemplateName())
                            .setContractTemplatePdf(contractTemplate.getContractTemplatePdf())
                            .setContractType(contractTemplate.getContractType())
                            .setContractStatus(SignContractStatusEnum.WAIT_SIGN.getKey())
                            .setValidDate(contractTemplate.getValidDate())
                            .setExpireDate(contractTemplate.getExpireDate())
                            .setContractTemplateId(contractTemplate.getContractTemplateId())
                            .setFadadaTemplateId(contractTemplate.getFadadaTemplateId())
                            .setNeedCompanySign(contractTemplate.getNeedCompanySign())
                            .setNeedManagerSign(contractTemplate.getNeedManagerSign())
                            .setNeedPartyASign(contractTemplate.getNeedPartyASign())
                            .setCaCustomerId(signItem.getCaCustomerId())
                            .setCompanySignFiled(contractTemplate.getCompanySignFiled())
                            .setManagerSignField(contractTemplate.getManagerSignField())
                            .setPartyASignFiled(contractTemplate.getPartyASignFiled());
                    signItemList.add(contractSignItem);
                }
                if (CollectionUtil.isNotEmpty(signItemList)) {
                    // 遍历网点合同签约明细找出填充字段配置
                    for (WebsitContractSignItem contractSignItem : signItemList) {
                        final List<ContractTemplateField> templateFieldList = contractTemplateFieldService.lambdaQuery()
                                .eq(ContractTemplateField::getContractTemplateId, contractSignItem.getContractTemplateId())
                                .list();
                        for (ContractTemplateField templateField : templateFieldList) {
                            WebsitContractSignField field = new WebsitContractSignField();
                            field.setWebsitContractSignRecordId(contractSignItem.getWebsitContractSignRecordId())
                                    .setWebsitContractSignItemId(contractSignItem.getId())
                                    .setFieldType(templateField.getFieldType())
                                    .setFieldName(templateField.getFieldName())
                                    .setFieldValue(templateField.getFieldValue());
                            signFieldList.add(field);
                        }
                    }
                }

                if (CollectionUtil.isNotEmpty(signItemList)) {
                    websitContractSignItemService.saveBatch(signItemList);

                    websitContractSignRecordService.lambdaUpdate()
                            .set(WebsitContractSignRecord::getStatus, SignContractStatusEnum.WAIT_SIGN.getKey())
                            .in(WebsitContractSignRecord::getId, signItems.stream()
                                    .map(WebsitContractSignItem::getWebsitContractSignRecordId)
                                    .collect(Collectors.toList()))
                            .in(WebsitContractSignRecord::getStatus, SignContractStatusEnum.WAIT.getKey(), SignContractStatusEnum.OK.getKey())
                            .update();
                }
                if (CollectionUtil.isNotEmpty(signFieldList)) {
                    websitContractSignFieldService.saveBatch(signFieldList);
                }

            }
        }
    }

    @Transactional
    public void update(ContractTemplateBean contractTemplate) {
        this.validRequireParams(contractTemplate);
        this.checkMainDate(contractTemplate);
        ContractTemplate byId = contractTemplateService.getById(contractTemplate.getContractTemplateId());
        if (!byId.getContractType().equals(contractTemplate.getContractType())) {
            throw new RemoteServiceException("不能修改合同类型");
        }

        if (!byId.getContractTemplatePdf().equals(contractTemplate.getContractTemplatePdf())) {
            String urlWw = contractTemplate.getContractTemplatePdf();
            String templateId = IdWorker.getIdStr();
            Auth4FDDUtil.uploadTemplate(templateId, null, urlWw);
            byId.setFadadaTemplateId(templateId);
        }

        byId.setContractTemplateName(contractTemplate.getContractTemplateName().trim())
                .setValidDate(contractTemplate.getValidDate())
                .setExpireDate(contractTemplate.getExpireDate())
                .setContractTemplatePdf(contractTemplate.getContractTemplatePdf())
                .setFollowContractTemplateId(contractTemplate.getFollowContractTemplateId())
                .setNeedPartyASign(StringUtils.isBlank(contractTemplate.getCompanySignFiled()) ? IsYesNoEnum.NO.getKey() : IsYesNoEnum.YES.getKey())
                .updateById();

        if (contractTemplate.getContractType().equals(ContractTypeEnum.MAIN.getKey())) {
            contractTemplateService.lambdaUpdate()
                    .set(ContractTemplate::getValidDate, byId.getValidDate())
                    .set(ContractTemplate::getExpireDate, byId.getExpireDate())
                    .eq(ContractTemplate::getFollowContractTemplateId, contractTemplate.getContractTemplateId())
                    .update();
        }

        contractWebsitService.lambdaUpdate()
                .eq(ContractWebsit::getContractTemplateId, contractTemplate.getContractTemplateId())
                .remove();
        if (contractTemplate.getContractType().equals(ContractTypeEnum.MAIN.getKey())
                && CollectionUtil.isNotEmpty(contractTemplate.getItems())) {
            contractWebsitService.saveBatch(contractTemplate.getItems().stream()
                    .peek(v -> v.setContractTemplateId(byId.getContractTemplateId())
                            .setCompanyWechatId(byId.getCompanyWechatId())
                            .setCompanyWechatName(byId.getCompanyWechatName())
                            .setId(IdWorker.getIdStr()))
                    .collect(Collectors.toList()));
        }

        contractTemplateFieldService.lambdaUpdate()
                .eq(ContractTemplateField::getContractTemplateId, contractTemplate.getContractTemplateId())
                .remove();
        if (CollectionUtil.isNotEmpty(contractTemplate.getFieldItems())) {
            contractTemplateFieldService.saveBatch(contractTemplate.getFieldItems().stream()
                    .peek(v -> v.setContractTemplateId(byId.getContractTemplateId()))
                    .collect(Collectors.toList()));
        }
    }

    public void delete(String contractTemplateId) {
        ContractTemplate template = contractTemplateService.lambdaQuery()
                .eq(ContractTemplate::getContractTemplateId, contractTemplateId)
                .one();
        ContractTemplateBak contractTemplateBak = new ContractTemplateBak();
        BeanUtils.copyProperties(template, contractTemplateBak);
        template.deleteById();
        contractTemplateBak.insert();
    }

    public void updateStatus(String contractTemplateId, StateEnum status) {
        ContractTemplateBean bean = new ContractTemplateBean();
        ContractTemplate template = contractTemplateService.lambdaQuery()
                .eq(ContractTemplate::getContractTemplateId, contractTemplateId)
                .one();
        BeanUtils.copyProperties(template, bean);
        final List<ContractWebsit> websitList = contractWebsitService.lambdaQuery()
                .eq(ContractWebsit::getContractTemplateId, template.getContractTemplateId())
                .list();

        final List<ContractWebsitBean> websitBeanList = BeanUtil.copyToList(websitList, ContractWebsitBean.class);
        bean.setItems(websitBeanList);

        if (status.getKey().equals(StateEnum.ON.getKey())) {
            this.checkMainDate(bean);
        }
        ContractTemplate contractTemplate = new ContractTemplate();
        contractTemplate.setStatus(status.getKey())
                .setContractTemplateId(contractTemplateId)
                .updateById();
    }


    private void validRequireParams(ContractTemplateBean contractTemplate) {
        if (StringUtils.isBlank(contractTemplate.getContractTemplateName())) {
            throw new RemoteServiceException("合同名称不能为空");
        }
        if (StringUtils.isBlank(contractTemplate.getContractType())) {
            throw new RemoteServiceException("合同类型不能为空");
        }
        if (contractTemplate.getContractType().equals(ContractTypeEnum.MAIN.getKey())) {
            contractTemplate.setFollowContractTemplateId(null);
        }
        if (StringUtils.isBlank(contractTemplate.getContractTemplateId())
                && ContractTypeEnum.MAIN.getKey().equals(contractTemplate.getContractType())
                && Objects.isNull(contractTemplate.getValidDate())) {
            throw new RemoteServiceException("主合同类型生效日期不能为空");
        }
        if (StringUtils.isBlank(contractTemplate.getContractTemplateId())
                && ContractTypeEnum.MAIN.getKey().equals(contractTemplate.getContractType())
                && Objects.isNull(contractTemplate.getExpireDate())) {
            throw new RemoteServiceException("主合同类型失效日期不能为空");
        }
        if (ContractTypeEnum.MAIN.getKey().equals(contractTemplate.getContractType())
                && contractTemplate.getValidDate().after(contractTemplate.getExpireDate())) {
            throw new RemoteServiceException("生效日期不能大于失效日期");
        }
        if (StringUtils.isNotBlank(contractTemplate.getCompanySignFiled())) {
            contractTemplate.setNeedCompanySign(IsYesNoEnum.YES.getKey());
        }
        if (StringUtils.isNotBlank(contractTemplate.getManagerSignField())) {
            contractTemplate.setNeedManagerSign(IsYesNoEnum.YES.getKey());
        }
        if (StringUtils.isNotBlank(contractTemplate.getPartyASignFiled())) {
            contractTemplate.setNeedPartyASign(IsYesNoEnum.YES.getKey());
        }
        if (StringUtils.isBlank(contractTemplate.getContractTemplateId())) {
            Integer count = contractTemplateService.lambdaQuery()
                    .eq(ContractTemplate::getContractTemplateName, contractTemplate.getContractTemplateName().trim())
                    .count();
            if (count > 0) {
                throw new RemoteServiceException("合同名称已存在，不能重复");
            }
        }
        if (StringUtils.isBlank(contractTemplate.getContractTemplatePdf())) {
            throw new RemoteServiceException("合同模板不能为空");
        }
        if (StringUtils.isNotBlank(contractTemplate.getFollowContractTemplateId())
                && ContractTypeEnum.FOLLOW.getKey().equals(contractTemplate.getContractType())) {
            final ContractTemplate mainContract = contractTemplateService.getById(contractTemplate.getFollowContractTemplateId());
            if (Objects.nonNull(mainContract)) {
                contractTemplate.setValidDate(mainContract.getValidDate())
                        .setExpireDate(mainContract.getExpireDate());
            }
        }
    }

    private void checkMainDate(ContractTemplateBean contractTemplate) {
        if (!contractTemplate.getContractType().equals(ContractTypeEnum.MAIN.getKey())) {
            return;
        }
        if (CollectionUtil.isEmpty(contractTemplate.getItems())) {
            return;
        }
        List<String> ids = commonMapper.selectMainContractIdByValidDate(contractTemplate.getCompanyWechatId(), contractTemplate.getValidDate());
        if (StringUtils.isNotBlank(contractTemplate.getContractTemplateId())
                && CollectionUtil.isNotEmpty(ids)) {
            ids.remove(contractTemplate.getContractTemplateId());
        }
        if (CollectionUtil.isNotEmpty(ids)) {
            for (ContractWebsitBean item : contractTemplate.getItems()) {
                final Integer count = contractWebsitService.lambdaQuery()
                        .in(ContractWebsit::getContractTemplateId, ids)
                        .in(ContractWebsit::getSubWebsitId, item.getSubWebsitId())
                        .count();
                if (count > 0) {
                    throw new RemoteServiceException(item.getSubWebsitId() + "的生效日期在" + String.join(",", ids) + "其他主合同的有效期内，请检查");
                }
            }
        }
    }

    public Map<String, Object> getField() {
        return FieldTypeEnum.websitList();
    }
}
